% Programm 10.6 Test der starken Konvergenz Euler-Maruyama
% am Beispiel dX = lambda X dt + mu X dW, X(0) = Xzero ,
% lambda = 2 , mu = 1 and Xzer0 = 1 .
%
% Brownsche Bewegung ueber [0,1], h = 2^(-9).
% 5 EM Schritte: 16 h , 8 h , 4 h , 2 h , h .
% Auswertung bei T=1: E | X_L - X(T) | .
randn('state',100) % random seeding
lambda = 2 ; mu = 1 ; Xzero = 1 ; 
T = 1 ; N = 2^9 ; h = T/N; %
M = 1000 ; % Zahl der gesamplten Pfade
Xerr = zeros(M,5) ; % Pre-Allokierung
dW = zeros(1,N); W = zeros(1,N); % Pre-Allokierung
for s = 1:M, % samples ueber approximierte Wiener-Prozess-Pfade
  dW = sqrt(h)*randn(1,N) ; % Inkremente
  W = cumsum(dW) ; % approximierter Wiener-Prozess-Pfad
  Xtrue = Xzero*exp((lambda -0.5*mu^2)+mu*W(end) ) ;
  for p = 1:5
    R = 2^(p-1); Dt = R*h ; L = N/R; % EM Schritte Dt = R h 
    Xtemp = Xzero ;
    for j = 1:L
      Winc = sum(dW(R*(j-1)+1:R*j)) ;
      Xtemp = Xtemp + Dt*lambda*Xtemp + mu*Xtemp*Winc ;
    end
    Xerr(s,p) = abs(Xtemp - Xtrue) ; % Fehler bei T=1
  end
end
Dtvals = h*(2.^([0:4])) ;
loglog(Dtvals,mean(Xerr),'b*'), hold on
loglog(Dtvals,(Dtvals.^(.5)),'g--') % Referenzsteigung von 1/2
axis([1e-3 1e-1 1e-2 1e+1 ])
xlabel('h','FontSize',16),ylabel('Mittel von |X(T) - X_L|','FontSize',16)
% Ausgleichskurve:  C*Dt^q 
A = [ones(5,1),log(Dtvals)'] ; rhs = log(mean(Xerr)');
sol = A\rhs ; q = sol(2)
C = exp(sol(1))
loglog(Dtvals,C*(Dtvals.^q),'r--'), hold off % tatsaechliche Steigung
resid = norm(A*sol - rhs)
% Programmende
