% Programm 10.5 em_milstein_allgemein_vs_exakt_gb.m ; 
% Euler-Maruyama und Milstein-Methode zur numerischen Loesung von
% SDE: dX = (alpha + mu*X) dt + (peta + sigma*X) dW, X(0) = Xzero ,
% alpha = 5 , mu = -2 , peta = 4 , sigma = 1 und Xzero = 1 .
%
% im Vergleich zur exakten Loesung
%
% Wiener-Prozess ueber [0,1], dt = 2^(-8).
% Euler-Maruyama, Milstein auf einem Gitter mit Dt = R*dt 
randn('state',100) % random seeding
alpha = 5 ; mu = -2 ; peta = 4 ; sigma = 1 ; Xzero = 1 ; % Problemparameter
T = 1 ; N = 2^8 ; dt = 1/N;
dW = zeros(1,N); W = zeros(1,N); % Pre-Allokierung
dW = sqrt(dt)*randn(1,N) ; % Inkremente des Wiener-Prozesses
W = cumsum(dW) ; % Wiener-Prozess
% Aufbau der exakten Loesung
for j = 1:N
   ct1 = 0;
   for k = 1:j
   summa = exp( (0.5*sigma^2-mu)*k*dt - sigma*W(k) );
   ct1 = ct1 + summa*dt;
   ct2(k) = exp( (0.5*sigma^2-mu)*k*dt - sigma*W(k) );
   end
   wito = ito_integral_gb(ct2(1:j),dW(1:j),j);
   Xtrue(j) = ( Xzero+exp(sigma*W(1))*( (alpha-peta*sigma)*ct1+peta*wito ) )...
              *exp( (mu-0.5*sigma^2)*j*dt+sigma*(W(j)-W(1)) );
end 
plot([0:dt:T],[Xzero,Xtrue],'g-'), hold on
xlabel('t','FontSize',16); ylabel('X_t','FontSize',16);
R = 4 ; Dt = R*dt ; L = N/R; % L Schritte der numerischen Verfahren Dt = R*dt
Xem = zeros(1,L); Yem = zeros(1,L); % Pre-Allokierung
Xtemp = Xzero ;
Ytemp = Xzero ;
for j = 1:L
   Winc = sum(dW(R*(j-1)+1:R*j)) ;
   Xtemp = Xtemp + Dt*(alpha+mu*Xtemp)+(peta+sigma*Xtemp)*Winc ;
   Xem(j) = Xtemp ;   %   Euler-Maruyama-Loesung
   Ytemp = Ytemp+Dt*(alpha+mu*Ytemp)+(peta+sigma*Ytemp)*Winc...
                +0.5*sigma*(peta+sigma*Ytemp)*(Winc^2-Dt);
   Yem(j) = Ytemp ;   %   Milstein-Loesung
end
plot([0:Dt:T],[Xzero,Xem],'r--*'), hold off
figure(2)
plot([0:dt:T],[Xzero,Xtrue],'g-'), hold on
plot([0:Dt:T],[Xzero,Yem],'b--*'), hold off
xlabel('t','FontSize',16); ylabel('X_t','FontSize',16);
eerrx = abs(Xem(end)-Xtrue(end))
merry = abs(Yem(end)-Xtrue(end))
% Programmende
