% Loesung der Schwingungsgleichung x'' + sin(x) = 0, x(0) = 1, x'(0) = 0
% Vergleich mit der exakten Loesung von der Approximationx'' + x = 0
% BDF2-Verfahren, verwendete Programme: bdf2_gbo.m, bdf2step_fo.m, f0.m

t0 = 0;
tf = 2*pi;
h = 1e-2;
x0 = [1.0,0.0];
x1 = [cos(h),sin(h)];
[x,t] = bdf2_gbo(t0,tf,x0,x1,h);

% berechne exakte Loesung unter der Annahme sin(x) ~ x fuer kleine x-Werte
nn = size(t);
xx = ones(nn,size(x0));
xx(:,1) = cos(t(:));
xx(:,2) = -sin(t(:));

% graphische Darstellung der numerischen Loesung
hf = figure(1)
plot(t, x(:,1), "-b", 'linewidth',3)
ax = gca();
set(ax, 'fontsize', 15);
hold on
plot(t, xx(:,1), "-r", 'linewidth',3)
ny = ylabel('{\bf y}')
set(ny,'fontsize',17)
nx = xlabel('{\bf x}')
set(nx,'fontsize',17)
nl = legend("num. Loesung","ex. Loesung")
set(nl,'fontsize',17)
legend("boxoff")
set (hf, "visible", "off");
print (hf, "oszi_f.png")
set (hf, "visible", "on");
system ("oszi_f");
open ("oszi_f.png");

%
% Plot der Ableitungen
hf = figure(2)
plot(t, x(:,2), "-b", 'linewidth',3)
h=get(gcf, "currentaxes");
set(h, "fontsize", 12, "linewidth", 2);
set(gca,'linewidth',3,'fontsize',16);
hold on
plot(t, xx(:,2), "-r", 'linewidth',3)
ny = ylabel('{\bf y)}')
set(ny,'fontsize',17)
nx = xlabel('{\bf x}')
set(nx,'fontsize',17)
nl = legend("num. Loesung","ex. Loesung")
set(nl,'fontsize',17)
legend("boxoff")
set (hf, "visible", "off");
print (hf, "oszi_a.png")
set (hf, "visible", "on");
system ("oszi_a");
open ("oszi_a.png");
