function [x,t] = radauIIA_dae(t0,tf,x0,h,daetype,parameter)
% Programm 9.2 (radauIIA.m)
% zur Loesung von DAEs f((d(x,t))',x,t)=0
% 2-stufiges Radau-IIA-Verfahren (Ordnung 3)
% input:  Intervallgrenzen t0,tf,
%         Anfangswert x0=x(t0), Schrittweite h
%         daetype ('proper' or 'classic'),
%         parameter für Parameter von f und d
% output: Loesung x(t) an den Punkten t
% benutztes Programm: radauIIAstep_fo
% Aufruf: [x,t] = radauIIA_dae(t0,tf,x0,h,daetype,parameter)

    n = floor((tf - t0)/h);
	[r,m] = size(x0);

    t = zeros(n+1,1); x = zeros(n+1,m);
    t(1) = t0; x(1,:) = x0;
    xk = x0;
	tk = t0;

    options = optimset('Display','off');
    for k = 1:n
        % berechne Nullstelle von radauIIAstep_f
        % und nehme (x_k,x_k) als Startnaeherung:
        Xstart = [xk xk];
        fun = @(x)radauIIAstep_fo(x,xk,tk,h,daetype,parameter);
        X = fsolve(fun,Xstart,options);
        xkp1 = X(m+1:2*m); % Loesung der letzten Stufe s=2
        tk = tk+h;       % update t_{k} fuer naechsten Schritt
        xk = xkp1;       % update x_{k} fuer naechsten Schritt
        t(k+1) = tk;     % speichere aktuellen Zeitpunkt t_{k+1}
        x(k+1,:) = xk;   % speichere aktuelle Loesung x_{k+1}
    end
end
