function F = radauIIAstep_fo(X,xk,tk,h,daetype,parameter)
% Die Stufenloesungen X=(X1,X2) eines zweistufigen Radau-IIA-Schrittes
% sind eine Nullstelle von radauIIAstep_fo.
%
% input:  X=[X1 X2] Startnaeherung für die Stufen
%         x_k Loesung des vorherigen Radau-IIA-Schrittes,
%         t_k vorheriger Zeitpunkt, h Schrittweite,
%         daetype ('proper' or 'classic'),
%         parameter für Parameter von f und d
% output: F Funktionswert der Nullstellenfunktion
%
% benutzte Programme: Nutzerfunktionen f und d
% der Form f = f(y,x,t,daetype,parameter)
% und d = d(x,t,daetype,parameter)
%
% Aufruf: F = radauIIAstep_fo(X,xk,tk,h,daetype,parameter)

    [r,zwei_m] = size(X);
    m = zwei_m/2;
    beta11 = 1.5;  beta12 = 0.5; 
    beta21 = -4.5; beta22 = 2.5;
    a1 = 1.0/3.0; a2 = 1.0;
    X1 = X(1:m);
    X2 = X(m+1:2*m);
    dX1 = dae_do_GearPetzold(X1,tk+a1*h,daetype,parameter);
    dX2 = dae_do_GearPetzold(X2,tk+a2*h,daetype,parameter);
    dxk = dae_do_GearPetzold(xk,tk,daetype,parameter);
    Y1 = (beta11*(dX1-dxk) + beta12*(dX2-dxk))/h;
    Y2 = (beta21*(dX1-dxk) + beta22*(dX2-dxk))/h;
    F1 = dae_fo_GearPetzold(Y1,X1,tk+a1*h,daetype,parameter);
    F2 = dae_fo_GearPetzold(Y2,X2,tk+a2*h,daetype,parameter);
    F = [F1 F2];
end 
