% Programm 9.4: slopelim_gb1.m, Loeser der Advektionsgleichung u_t + au_x = 0
% Methoden : 'Lax-Wendroff/lw' (Standard Lax-Wendroff), 
%            'Lax-Wendroff-sl/lwsl' (Lax-Wendroff mit minmod-slope-Limiter). 
% Anfangsbedingung:  1 = stueckw. konst. (Shock), 2 = stueckw. konst. (Expansion)
% Beispielaufruf: slopelim_gb1("Lax-Wendroffsl",100,0.01,1)
function slopelim_gb1( method, nx, dt, ictype )
a = 1.;
if nargin < 1,
  disp('Benutzung:  slopelim_gb( method, nx, dt, ictype )');
  disp([' ''Lax-Wendroff-sl/Lax-Wendroff-sl'', ''Lax-Wendroff/Lax-Wendroff''']);
  return;
end
tend = 4;xmax = 8;     % Endzeit, raeumliches Intervall [0,xmax]
dx = xmax/nx % Raumschrittweite
x  = [0 : dx : xmax]; nt = floor(tend/dt); dt = tend / nt;
ntprint = 50; % Zeitschrittabstand der Plots der Profile
% Anfangswert
u0   = uinit(x,ictype); u    = u0; unew = 0*u;
disp(['Method: ',method]);disp([' dx=',num2str(dx)]);disp([' dt=',num2str(dt)]);
ntprint = min(nt, ntprint); dtprint = tend / ntprint;
% Loesung im Zeit-Raum-Gebiet.
uall = zeros(ntprint+1,nx+1); uall(1,:) = u0;
ip = 1;
figure(1)
for i = 1 : nt,
  t = i*dt;
  switch lower(method)
   case {'lax-wendroffsl','Lax-Wendroffsl'}
%  slopes - minmod-Limiter
    for j=2:nx,
		dux = (u(j) -u(j-1))/dx; duxp = (u(j+1)-u(j))/dx;
		if abs(dux) < abs(duxp) && dux*duxp > 0 
			sigma(j) = dux;
		elseif abs(dux) > abs(duxp) && dux*duxp > 0
			sigma(j) = duxp;
		else
			sigma(j) = 0;
		end
    end
    sigma(1) = sigma(2);
	unew(2:nx) = u(2:nx) -a*dt/dx*(u(2:nx)-u(1:nx-1)) -a*dt/2*(sigma(2:nx)-sigma(1:nx-1)) ...
		+ 0.5*(a*dt)^2/dx*(sigma(2:nx)-sigma(1:nx-1));
    unew(1)   = u(1); unew(end) = u(end);
   case {'lax-wendroff','Lax-Wendroff'}
%  slopes - Standard
	sigma(1:nx) = (u(2:nx+1)-u(1:nx))/dx;
	unew(2:nx) = u(2:nx) -a*dt/dx*(u(2:nx)-u(1:nx-1)) -a*dt/2*(sigma(2:nx)-sigma(1:nx-1)) ...
		+ 0.5*(a*dt)^2/dx*(sigma(2:nx)-sigma(1:nx-1));
    unew(1)   = u(1); unew(end) = u(end);
   otherwise  % upwind (default)
	unew(2:nx) = u(2:nx) - a*dt/dx*(u(2:nx)-u(1:nx-1));
    unew(1)    = u(1); unew(end) = u(end);
  end  % switch
% Plot der Profile, alle ntprint Zeitschritte
  if t >= ip*dtprint,
    plot(x, unew)
    xlabel('x'), ylabel('u')
    title( [method, ', Loesung zum Zeitpunkt t=', num2str(t,'%9.4f')] )
    grid on, shg
    pause(0.1)
    ip = ip + 1; uall(ip,:) = unew;
  end
  u = unew;
end
end
%
function ui = uinit( x, ictype )
xshift = 1.0;
switch ictype
 case 1    % shock (uL > uR)
  uL = 1.0; uR = 0.5; ui = uR + (uL-uR) * ((x-xshift) <= 0.0);
 case 2    % expansion (uL < uR)
  uL = 0.5; uR = 1.0; ui = uR + (uL-uR) * ((x-xshift) <= 0.0);
end % switch
end
