function [x,t] = radauIIA_gb(t0,tf,x0,h,daetype,parameter)
% Programm 9.1 (radauIIA.m) zur Loesung der DAE f((d(x,t))',x,t)=0
% 2-stufiges Radau-IIA-Verfahren (Ordnung 3)
% input:  Intervallgrenzen t0,tf; Anfangswert x0=x(t0); Schrittweite h
% output: Loesung x(t) an den Punkten t
% benutzte Programme: Nutzerfunktionen radauIIA_step_gb, fdae_gb und ddae_gb
% Aufruf: [x,t] = radauIIA_gb(t0,tf,x0,h,daetype,parameter)

n = floor((tf - t0)/h);
m0 = size(x0);
m = m0(1);

t = zeros(n+1,1); x = zeros(n+1,m);
t(1) = t0; x(1,:) = x0;
xk = x0; % starte mit x_k:=x0
tk = t0; % aktueller Zeitpunkt t_k

beta11 = 1.5;  beta12 = 0.5;
beta21 = -4.5; beta22 = 2.5;
a1 = 1.0/3.0; a2 = 1.0;

options = optimset('Display','off');
for k = 1:n
	% berechne Nullstelle von radauIIAstep_f
	% und nehme (x_k,x_k) als Startnaeherung:
	Xstart = [xk; xk];
	fun = @(X)radauIIA_step_gb(X,xk,tk,h,daetype,parameter,m,a1,a2,beta11,beta12,beta21,beta22);
	X = fsolve(fun,Xstart,options);
	xkp1 = X(m+1:2*m); % Loesung der letzten Stufe s=2

	tk = tk+h;       % update t_k fuer naechsten Schritt
	xk = xkp1;       % update x_k fuer naechsten Schritt
	t(k+1) = tk;     % speichere aktuellen Zeitpunkt t_{k+1}
	x(k+1,:) = xk;   % speichere aktuelle Loesung x_{k+1}
end
end
